// 22/7/95 Ramn David Talavera Suarez
// Header file for matrix type management
// notation ROW-COL
// first ROW 0
// fist COL 0
// needs c++ to compile
// Just use replace option to pass from float matrix to double,..,int.

#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <math.h>
#include <dos.h>


#define ALMOST_CERO             1e-5    //if number<ALMOST_CERO it is DISPLAYED as a 0
#define DISPLAY_SEP_FACTOR_X    6       // display separator factor.
#define DISPLAY_SEP_FACTOR_Y    2
#define WATCOMC       0                 // Set to 1 if you are compiling in WATCOM C

#if !WATCOMC
	#include <conio.h>
#endif

class matrix {
	private:
	      unsigned int ROWS,COLS;
	public:
	      float **point;

	protected:
	      float Det2x2();

	public:
	      matrix (unsigned int rows,unsigned int cols);                     // CONSTRUCTOR
	      matrix(const matrix &REF);                                // COPY CONSTRUCTOR
	      ~matrix();                                                // DESTRUCTOR

	      matrix operator + (matrix &B);
	      matrix operator - (matrix &B);
	      matrix operator * (matrix &B);
	      matrix operator !();                                      // TRANSPOSE (unary)
	      matrix operator =(matrix &B);                             // EQUALITY!

	     inline matrix operator * (float a);                        //hope it puts it inline!
	     inline matrix operator / (float a);
	     inline matrix operator * (int a);
	     inline matrix operator / (int a);

	     inline matrix set_cero();

	      friend float Det(matrix &B);                              // Det()

	      friend matrix Adj(matrix &B);                             // Adj()
	      friend matrix Inv(matrix &B);                             // Inv()
	      matrix Menor (unsigned int a,unsigned int b);                     // Menor ( row ,col )

	      void display (unsigned int x1,unsigned int y1);           //display matrix at x1,y1
	      matrix input();                                   // inputs matrix

	      inline matrix swap_rows(unsigned int row1,unsigned int row2);
	      inline matrix swap_cols(unsigned int col1,unsigned int col2);

	// LU functions======> FAAST INVERSE&DET CALCULATIONS
	      friend matrix ass_LU(matrix &B,int *indx,float *signo);   //(associated LU matrix)
	      friend void LU_bksb(matrix &A,float *vector,int *indx);  // solves the ecuation A*X=B, the solution comes into X
	      // look matrix.txt for a brief explanation.
	public:
	      friend matrix F_Inv(matrix &B);
	      friend float F_Det(matrix &D);

	      }  ;

void wait_key(char key);

	/***********************
	**     #WATCOM C#     **
	**   gotoxy & clrscr  **
	************************/

/* Next lines should be enabled for the WATCOM C compiler
	and should be disabled for Borland C */
#if WATCOMC
void gotoxy( unsigned int col, unsigned int row )
{
#ifdef __OS2__
    VioSetCurPos( row, col, 0 );
#else
    union REGS                  regs;

    regs.w.dx = (unsigned short)(( row << 8 ) + col - 0x0101);
    regs.h.bh = 0;
    regs.h.ah = 2;
#ifdef __386__
    int386( 0x10, &regs, &regs );
#else
    int86( 0x10, &regs, &regs );
#endif
#endif
}

void clrscr()
{
#ifdef __OS2__
    VioWrtNChar( " ", 2000, 0, 0, 0 );
#else
    union REGS                  regs;

    regs.w.cx = 0;
    regs.w.dx = 0x1850;
    regs.h.bh = 7;
    regs.w.ax = 0x0600;
#ifdef __386__
    int386( 0x10, &regs, &regs );
#else
    int86( 0x10, &regs, &regs );
#endif
#endif
gotoxy(1,1);
}
#endif //WATCOMC


	/***********************
	**   CONSTRUCTOR      **
	************************/

matrix::matrix(unsigned int rows,unsigned int cols)
	{
	unsigned int i;
	ROWS=rows;COLS=cols;
	if ( !( point = new float*[ROWS] ) )
	 {
	 cerr << "Couldn't allocate the matrix in memory";
	 exit(EXIT_FAILURE);
	 }
	 for (i=0;i<ROWS;i++)
	  if ( !( point [i]=new float[COLS] ) )
	   {
	   cerr << "Couldn't allocate the matrix in memory";
	   exit(EXIT_FAILURE);
	   }
	}

	/**************************
	**  COPY CONSTRUCTOR     **
	***************************/

matrix::matrix(const matrix &REF)
	{
	unsigned int i,j;
		// allocate new matrix
	ROWS=REF.ROWS;COLS=REF.COLS;
	if ( !( point = new float*[ROWS] ) )
	 {
	 cerr << "Couldn't allocate the matrix in memory";
	 exit(EXIT_FAILURE);
	 }
	for (i=0;i<ROWS;i++)
	  if ( !( point [i]=new float[COLS] ) )
	   {
	   cerr << "Couldn't allocate the matrix in memory";
	   exit(EXIT_FAILURE);
	   }
		 // copy reference matrix to new matrix
		// possibly faster when using memcpy(..)
	 for (j=0;j<COLS;j++)
	  for (i=0;i<ROWS;i++)
	   point[i][j]=REF.point[i][j];
	}





	/***********************
	**      EQUALITY (=)  **
	************************/

/*matrix matrix::operator=(matrix &B) // FIRST VERSION=>OBSOLETE
	{
	// possibly faster using memcpy (less readable code)
	unsigned int i,j;
	if ( (B.ROWS!=ROWS)||(B.COLS!=ROWS) )
		{
		cerr <<"Dimensions aren't equal, equality impossible\n";
		exit (EXIT_FAILURE);
		}
	for (j=0;j<COLS;j++)
	  for (i=0;i<ROWS;i++)
	     point[i][j]=B.point[i][j];
	  return *this;
	 }*/


matrix matrix::operator=(matrix &B)
	{
	unsigned int i,j;
	for (i=0;i<COLS;i++)
	  delete(point[i]);
	delete (point);

	ROWS=B.ROWS;COLS=B.COLS;

	if ( !( point = new float*[ROWS] ) )
	 {
	 cerr << "Couldn't allocate the matrix in memory";
	 exit(EXIT_FAILURE);
	 }
	 for (i=0;i<ROWS;i++)
	  if ( !( point [i]=new float[COLS] ) )
	   {
	   cerr << "Couldn't allocate the matrix in memory";
	   exit(EXIT_FAILURE);
	   }

	for (j=0;j<B.ROWS;j++)
	 for (i=0;i<B.COLS;i++)
	  point[i][j]=B.point[i][j];
	return *this;
	}

	/************************
	 **     DESTRUCTOR     **
	 ************************/
matrix::~matrix()
	{
	unsigned int i;
	for (i=0;i<ROWS;i++)
	  delete(point[i]);
	delete (point);
	}

	/***********************
	**    TRASPOSE (!)    **
	************************/

matrix matrix::operator !()
	{
	unsigned int i,j;
	matrix TEMP(ROWS,COLS);
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	   TEMP.point[j][i]=point[i][j];
	 return TEMP;
	 }

	/***********************
	**     SET TO CERO    **
	************************/

matrix matrix:: set_cero ()
	{
	// Better do it inline;
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	   point[i][j]=0;
	 return *this;
	 }

	/***********************
	**     MINUS   (-)    **
	************************/

matrix matrix :: operator -(matrix &B)
	{
	unsigned int i,j;
	if ((COLS!=B.COLS)||(ROWS!=B.ROWS))
		{
		cerr << "Matrix aren't compatible bellow -.";
		exit (EXIT_FAILURE);
		}
	matrix C(ROWS,COLS);
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	     {
	     C.point[i][j]=point[i][j]-B.point[i][j];
	     }
	 return C;
	 }









	/***********************
	**      ADDER   (+)   **
	************************/
matrix matrix :: operator +(matrix &B)
	{
	unsigned int i,j;

	if ((COLS!=B.COLS)||(ROWS!=B.ROWS))
		{
		cerr << "Matrix aren't compatible bellow +.";
		exit (EXIT_FAILURE);
		}
	matrix C(ROWS,COLS);
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	     {
	     C.point[i][j]=B.point[i][j]+point[i][j];
	     }
	 return C;
	 }

	/***********************
	**     MULTIPLY(*)    **
	************************/

matrix matrix :: operator * (matrix &B)
	{
	unsigned int i,j,k;
	if (COLS!=B.ROWS)
		{
		cerr << "Matrix aren't compatible bellow *.";
		exit (EXIT_FAILURE);
		}
	matrix M(ROWS,B.COLS);
	M.set_cero(); //inline:
		// Multiplication:
	for (j=0;j<B.COLS;j++)
	 for (i=0;i<ROWS;i++)
	   for (k=0;k<B.ROWS;k++)
	     M.point[i][j]=M.point[i][j]+point[i][k]*B.point[k][j];
	return M;
	}

	/***********************
	**    INVERSE Inv(M)  **
	************************/

matrix Inv(matrix &B)
	{
	unsigned int i,j;
	float det;
	matrix InvERSA(B.ROWS,B.COLS);
	det=Det(B);
	if (det==0)
	 {
	 cerr<<"Matrix has no Inverse";
	 exit(EXIT_FAILURE);
	 }
		// Inversa(A)=adjunto(traspuesta(A))/Det(A)
	  InvERSA=(Adj(!B))/det;
	 return InvERSA;
	}

	/***********************
	**  Det2x2(protected) **
	************************/

float matrix::Det2x2()
	{
	// it doesn`t have direct access, you must call Det(..) instead!
	float det;
	det=point[0][0]*point[1][1]-point[0][1]*point[1][0];
	return det;
	}


	/***********************
	**       Det (M)      **
	************************/

float Det(matrix &B)
	{
	unsigned int n;
	int signo;
	float det=0;

	if (B.ROWS!=B.COLS)
	 {
	 cerr<<"Matrix should be square!";
	 exit(EXIT_FAILURE);
	 }
	 else
	 if (B.ROWS==1)
	  return B.point[0][0];
	 else
	 if (B.ROWS==2)
	  return B.Det2x2();
	  else
	  for (n=0;n<B.COLS;n++)
	   {
	   //if ((n%2)==0) signo=1; else signo=-1;  // LOOK DOWN!, it's the same but using bitwise op.
	   (n&1)==0 ? (signo=1) : (signo=-1);  // checks parity: true->signo=+, false->signo=-
	   det=det+signo*B.point[0][n]*Det(B.Menor(0,n));
	   }
	return det;
	}










	/***********************
	**    MATRIX*number   **
	************************/

matrix matrix :: operator *(float a)
	{
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	  point[i][j]=a*point[i][j];
	return *this;
	}

matrix matrix :: operator *(int a)
	{
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	  point[i][j]=a*point[i][j];
	return *this;
	}

	/***********************
	**    MATRIX/number   **
	************************/

matrix matrix :: operator /(float a)
	{
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	  point[i][j]=point[i][j]/a;
	return *this;
	}

matrix matrix :: operator /(int a)
	{
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 for (i=0;i<ROWS;i++)
	  point[i][j]=point[i][j]/a;
	return *this;
	}

	/***********************
	**      MENOR(a,b)    **
	************************/

/*matrix matrix::Menor(unsigned int a,unsigned int b) =>OBSOLETE
	{
	unsigned int i,j;
	matrix MEN(ROWS-1,COLS-1);
	for (j=0;j<b;j++)
	 for (i=0;i<a;i++)
	   MEN.point[i][j]=point[i][j];
	for (j=0;j<b;j++)
	 for (i=(a+1);i<ROWS;i++)
	   MEN.point[i-1][j]=point[i][j];
	for (j=b+1;j<COLS;j++)
	 for (i=0;i<a;i++)
	   MEN.point[i][j-1]=point[i][j];
	for (j=b+1;j<COLS;j++)
	 for (i=a+1;i<ROWS;i++)
	   MEN.point[i-1][j-1]=point[i][j];
	 return MEN;
	}*/


matrix matrix::Menor(unsigned int a,unsigned int b)
	{
	unsigned int i,j,p,q;
	matrix MEN(ROWS-1,COLS-1);
	for (j=0,q=0;q<MEN.COLS;j++,q++)
	 for (i=0,p=0;p<MEN.ROWS;i++,p++)
	  {
	  if (i==a) i++;
	  if (j==b) j++;
	  MEN.point[p][q]=point[i][j];
	  }
	 return MEN;
	}


	/***********************
	**       ADJUNTA      **
	************************/

matrix Adj (matrix &B)
	{
	unsigned int i,j;
	float signo;
	 matrix ADJ(B.ROWS,B.COLS);
	for (j=0;j<B.COLS;j++)
	 for (i=0;i<B.ROWS;i++)
	  {
	  //if (((i+j)%2)==0) signo=1; else signo=-1;
	  ((i+j)&1)==0 ? (signo=1) : (signo=-1); // checks parity: true->signo=+, false->signo=-
	  ADJ.point[i][j]=signo*Det(B.Menor(i,j));
	  }
	  return ADJ;
	}















	/***********************
	**       DISPLAY      **
	************************/

void matrix :: display (unsigned int x1,unsigned int y1)
	{
	unsigned int i,j;
	for (j=0;j<COLS;j++)
	 {
	  for (i=0;i<ROWS;i++)
	    {
	    gotoxy( (i*DISPLAY_SEP_FACTOR_X)+x1 , (j*DISPLAY_SEP_FACTOR_Y)+y1 );
	    if ( fabs(point[j][i]) > ALMOST_CERO )
	     {
	     cout <<' '<<point[j][i]<<' ';
	     flush(cout);
	     }
	    else
	     {
	     cout <<' '<<0<<' ';
	     flush(cout);
	     }
	    }
	  cout<<'\n';
	  flush(cout);
	  }
	}

	/***********************
	**        INPUT       **
	************************/

matrix matrix::input()
	{
	unsigned int i,j;
	clrscr();
	cout << "Waiting for the ";
	cout << ROWS<< 'X'<<COLS<<" matrix data"<<'\n';
	flush(cout);
	  for (i=0;i<ROWS;i++)
	   for (j=0;j<COLS;j++)
	    {
	    cin>>point[i][j];
	    }
	  return *this;
	  }












	/***********************
	**    ROW&COL swaping **
	************************/

/*matrix matrix::swap_rows(unsigned int row1,unsigned int row2) =>OBSOLETE
	{
	// why do this when we can just change the pointers!!!
	unsigned int j;
	matrix FILA(1,COLS);
	if ( (row1>ROWS)||(row2>ROWS) )
		{
		cerr<<"Impossible to swap unexistent rows!";
		exit(EXIT_FAILURE);
		}
	for (j=0;j<COLS;j++)
		{
		FILA.point[0][j]=point[row1][j];
		point[row1][j]=point[row2][j];
		point[row2][j]=FILA.point[0][j];
		}
	return *this;
	}*/

matrix matrix::swap_rows(unsigned int row1,unsigned int row2)
	{
	unsigned int j;
	float *FILA=new (float);
	if ( (row1>ROWS)||(row2>ROWS) )
		{
		cerr<<"Impossible to swap unexistent rows!";
		exit(EXIT_FAILURE);
		}
	FILA=point[row1];
	point[row1]=point[row2];
	point[row2]=FILA;

	return *this;
	}

matrix matrix::swap_cols(unsigned int col1,unsigned int col2)
	{
	unsigned int i;
	float COLUMN;
	if ( (col1>COLS)||(col2>COLS) )
		{
		cerr<<"Impossible to swap unexistent cols!";
		exit(EXIT_FAILURE);
		}
	for (i=0;i<ROWS;i++)
		{
		COLUMN=point[i][col1];
		point[i][col1]=point[i][col2];
		point[i][col2]=COLUMN;
		}
	return *this;
	}



	/***********************
	**     LU FUNCTIONS   **
	************************/
/* These functions provide a VEEERY FAAAST matrix determinant and inverse calculations.
     (Normal way took 7 minutes to invert a 9x9 matrix, this way was direct) */

matrix ass_LU(matrix &B,int *indx,float *signo)
	{
	/* Crout's algorithm to give the associated LU matrix to a given matrix B, indx & signo
	keep trace of the row permutation and +-1 depending on whether the permutation number was
	even or odd, this is used to sign the determinant as its absolute value is equal to multiplying the     principal LU matrix diagonal elements*/
	unsigned int imax,i,j,k;
	float big,dum,sum,temp;
	float *vv;
	vv=new float[B.ROWS];

	if (B.ROWS!=B.COLS)
	 {
	 cerr<<"Matrix should be square!";
	 exit(EXIT_FAILURE);
	 }

	matrix LU(B.ROWS,B.COLS);
	LU=B;
	*signo=1.0;

	for (i=0;i<LU.COLS;i++)
	  {
	  big=0.0;
	  for (j=0;j<LU.COLS;j++)
		if ((temp=fabs(LU.point[i][j])) >big) (big=temp);
	  if (big==0.0)
		{
		cerr<<"singular matrix in routine ass-LU";
		exit(EXIT_FAILURE);
		}
		vv[i]=1.0/big;
	  }

	  for (j=0;j<LU.COLS;j++)
		{
		for (i=0;i<j;i++)
			{
			sum=LU.point[i][j];
			for (k=0;k<i;k++)
			 sum-=(LU.point[i][k])*(LU.point[k][j]);
			LU.point[i][j]=sum;
			}
		big=0.0;
		for (i=j;i<LU.COLS;i++)
			{
			sum=LU.point[i][j];
			for (k=0;k<j;k++)
			 sum-=LU.point[i][k]*LU.point[k][j];
			LU.point[i][j]=sum;
			if ( ( dum=vv[i]*fabs(sum) ) >=big )
				{
				big=dum;
				imax=i;
				}
			}
		if (j != imax)
		   {
		    LU.swap_rows(imax,j);
		    *signo=-(*signo);
		    vv[imax]=vv[j];
		    }
		indx[j]=imax;

		if (j!=LU.COLS)
			{
			if (LU.point[j][j]==0)
				{
				cerr<<"Impossible to calculate associated LU => Matrix has no inverse";
				exit (EXIT_FAILURE);
				}
			dum=1.0/(LU.point[j][j]);
			for (i=j+1;i<LU.COLS;i++)
				(LU.point[i][j])*=dum;
			}
		}
	  delete (vv);

	  return LU;
	  }

void LU_bksb(matrix &A,float *vector,int *indx)
	{
	/* Solves the equation A*X=B, the result is put into  X. */
	int i,ii=0,ip,j;
	float sum;

	for (i=0;i<A.ROWS;i++)
		{
		ip=indx[i];
		sum=vector[ip];
		vector[ip]=vector[i];
		if ((ii+1))
		 for (j=ii;j<=i-1;j++)
		  sum -=A.point[i][j]*vector[j];
		else if (sum) ii=i;
		vector[i]=sum;
		}
	for (i=(A.ROWS)-1;i>=0;i--)
		{
		sum=vector[i];
		for (j=i+1;j<A.COLS;j++)
			sum -=A.point[i][j]*vector[j];
		vector[i]=sum/A.point[i][i];
		}
	}



float F_Det(matrix &B)
	{
	/* Multiply the LU principal diagonal elements and multiply the result by it's associated sign*/
	unsigned int i;
	float det=1;
	int *indx;
	float *signo;
	indx=new int[B.ROWS];
	signo=new float;
	if (B.ROWS!=B.COLS)
		{
		cerr<<"Matrix should be square";
		exit(EXIT_FAILURE);
		}
	matrix LU(B.ROWS,B.COLS);
	LU=ass_LU(B,indx,signo);

	for (i=0;i<B.ROWS;i++)
	 det*=LU.point[i][i];

	det*=*signo;
	delete (indx);
	delete (signo);

	return det;
	}

matrix F_Inv(matrix &B)
	{
	unsigned int i,j;
	int *indx;
	float *signo;
	float *col;
	if (fabs(F_Det(B))<=ALMOST_CERO)
		{
		cerr<<"Not enough precission or matrix has no inverse";
		exit(EXIT_FAILURE);
		}
	indx=new int[B.ROWS];
	signo=new float;
	col=new float[B.ROWS];
	matrix LU=ass_LU(B,indx,signo);

	matrix INVERSE(B.ROWS,B.COLS);
	*signo=1;
	for (j=0;j<B.ROWS;j++)
		/* We now find the inverse by columns */
		{
		for(i=0;i<B.ROWS;i++) col[i]=0.0;
		col[j]=1.0;
		LU_bksb(LU,col,indx);
		for (i=0;i<B.ROWS;i++) INVERSE.point[i][j]=col[i];
		}

	delete (indx);
	delete (signo);
	return INVERSE;
	}

	/***********************
	**       WAIT KEY            **
	************************/

void wait_key(char key)
	{
	/* It magically waits for the given char to continue */ 
	char test='~';
	gotoxy(1,18);
	cout<<key<<"\n + enter to continue \n";
	flush(cout);
	while(test!=key)
	  cin>>test;
	}
